/*

------------------------------------------------------------------------------

A license is hereby granted to reproduce this software source code and
to create executable versions from this source code for personal,
non-commercial use.  The copyright notice included with the software
must be maintained in all copies produced.

THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE.  THE
AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE THE
INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.

Copyright (c) 1994-2006, John Conover, All Rights Reserved.

Comments and/or bug reports should be addressed to:

    john@email.johncon.com (John Conover)

------------------------------------------------------------------------------

tsunique.c is for removing duplicate records from a file. If the
records are a temporal time series, with time stamps as the first
field(s) of the records, the comparison for uniqueness can begin after
the time stamps. The unique records are printed to stdout.


Tsunique is for removing duplicate records from a file. If the records
are a temporal time series, with time stamps as the first field(s) of
the records, the comparison for uniqueness can begin after the time
stamps. The unique records are printed to stdout.

$Revision: 0.0 $
$Date: 2006/01/18 19:36:00 $
$Id: tsunique.c,v 0.0 2006/01/18 19:36:00 john Exp $
$Log: tsunique.c,v $
Revision 0.0  2006/01/18 19:36:00  john
Initial version


*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

static char rcsid[] = "$Id: tsunique.c,v 0.0 2006/01/18 19:36:00 john Exp $"; /* program version */
static char copyright[] = "Copyright (c) 1994-2006, John Conover, All Rights Reserved"; /* the copyright banner */

#define BUFLEN BUFSIZ /* i/o buffer size */

#ifdef __STDC__

static const char *help_message[] = /* help message index array */

#else

static char *help_message[] = /* help message index array */

#endif

{
    "\n",
    "Remove duplicate records from a file\n",
    "Usage: tsunique [-e preclipse] [-p] [-v] [filename]\n",
    "    -e preclipse, start comparison at column[preclipse], [0]\n",
    "    -p, output the preclipsed records, instead of entire records\n",
    "    -v, print the program's version information\n",
    "    filename, input filename\n"
};

#ifdef __STDC__

static const char *error_message[] = /* error message index array */

#else

static char *error_message[] = /* error message index array */

#endif

{
    "No error\n",
    "Error in program argument(s)\n",
    "Error opening file\n",
    "Error closing file\n"
};

#define NOERROR 0 /* error values, one for each index in the error message array */
#define EARGS 1
#define EOPEN 2
#define ECLOSE 3

#define PRECLIPSE 26 /* all characters in every record before the 26'th will be ignored */

#ifdef __STDC__

static void print_message (int retval); /* print any error messages */

#else

static void print_message (); /* print any error messages */

#endif

#ifdef __STDC__

int main (int argc, char *argv[])

#else

int main (argc, argv)
int argc;
char *argv[];

#endif

{
    char buffer[BUFLEN + 1], /* i/o buffer */
         oldbuffer[BUFLEN + 1], /* old i/o buffer */
         *new = &buffer[0], /* reference in i/o buffer */
         *old = &oldbuffer[0], /* reference in i/o buffer */
         *temp; /* buffer reference */

    int retval = NOERROR, /* return value, assume no error */
        preclipse = 0, /* character where comparison of duplicate records begins */
        preclipse_output = 0, /* output preclipsed record flag, 0 = no, 1 = yes */
        c; /* command line switch */

    FILE *infile = stdin; /* reference to input file */

    while ((c = getopt (argc, argv, "e:pv")) != EOF) /* for each command line switch */
    {

        switch (c) /* which switch? */
        {

            case 'p': /* request to output preclipsed records? */

                preclipse_output = 1; /* yes, set the output preclipsed record flag */
                break;

            case 'e': /* request for preclipse? */

                preclipse = atoi (optarg); /* yes, get the preclipse */

                if ((preclipse < 0) || (preclipse > BUFLEN)) /* valid preclipse? */
                {
                    retval = EARGS; /* assume argument error */
                }

                break;

            case 'v':

                (void) printf ("%s\n", rcsid); /* print the version */
                (void) printf ("%s\n", copyright); /* print the copyright */
                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */

            default: /* illegal switch? */

                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */
                break;
        }

    }

    if (retval == NOERROR) /* enough arguments? */
    {

        buffer[0] = '\0'; /* null the i/o buffer */
        oldbuffer[0] = '\0'; /* null the old i/o buffer */
        buffer[BUFLEN] = '\0'; /* null the i/o buffer sentinel */
        oldbuffer[BUFLEN] = '\0'; /* null the old i/o buffer sentinel */
        buffer[preclipse] = '\0'; /* null the preclipsed character in the i/o buffer */
        oldbuffer[preclipse] = '\0'; /* null the preclipsed character in the old i/o buffer */

        retval = EOPEN; /* assume error opening file */

        if ((infile = argc <= optind ? stdin : fopen (argv[optind], "r")) != (FILE *) 0) /* yes, open the input file */
        {
            retval = NOERROR; /* assume no error */

            while (fgets (new, BUFLEN, infile) != (char *) 0) /* read the records from the input file */
            {

                if (strcmp (&new[preclipse], &old[preclipse]) != 0) /* this record like the last one? */
                {

                    if (preclipse_output == 0) /* output preclipsed record flag, 0 = no, 1 = yes, set? */
                    {
                        (void) fputs (new, stdout); /* print the preclipsed record to stdout-no protection, since stdout is always available */
                    }

                    else
                    {
                        (void) fputs (&new[preclipse], stdout); /* print the entire record to stdout-no protection, since stdout is always available */
                    }

                }

                temp = new; /* swap the references */
                new = old;
                old = temp;
            }

            if (argc > optind) /* using stdin as input? */
            {

                if (fclose (infile) == EOF) /* no, close the input file */
                {
                    retval = ECLOSE; /* error closing file */
                }

            }

        }

    }

    print_message (retval); /* print any error messages */
    exit (retval); /* exit with the error value */

#ifdef LINT

    return (0); /* for lint formalities */

#endif

}

/*

Print any error messages.

static void print_message (int retval);

I) Data structures:

    A) The help_message array is an array of character stings, one per
       line to be printed for help.

    B) The error_message array is an array of character strings, one
       line per error; the error_message array is implicitly addressed
       by the value integer retval, which specifies the error to be
       printed.

II) Function execution:

    A) Depending on the value of the integer retval:

        1) If retval is zero, print nothing-normal/successful program
           exit.

        2) Else if retval is unity, print help.

        3) Else retval is not zero or unity, it is an error code, print
           the corresponding error message.

Returns nothing.

*/

#ifdef __STDC__

static void print_message (int retval)

#else

static void print_message (retval)
int retval;

#endif

{
    size_t help_ctr; /* help_message line counter */

    switch (retval) /* which return value */
    {

        case 0: /* program ended without errors, print nothing */

            break;

        case 1: /* program ended with a request for help, or argument error, print help */

            for (help_ctr = 0; help_ctr < (sizeof (help_message) / sizeof (char *)); help_ctr ++) /* for each line of help */
            {
                (void) printf ("%s", help_message[help_ctr]); /* print the line */
            }

            break;

        default: /* an error that was not a request for help, print the error */

            (void) fprintf (stderr, "%s", error_message[retval]);
            break;

    }

}
